'use strict';

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _lodash = require('lodash');

var _injectEnv = require('inject-env');

var _injectEnv2 = _interopRequireDefault(_injectEnv);

var _schema = require('./schema');

var _schema2 = _interopRequireDefault(_schema);

var _options2 = require('./schema/options');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var extensions = ['js', 'json'];

var jsonInjector = function jsonInjector() {
  var inputOptions = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

  var options = (0, _lodash.assign)(_options2.defaultOptions, inputOptions);

  var cwd = process.cwd();
  var rcFilePath = null;
  var rcFileExists = false;
  while (!(rcFileExists = _fsExtra2.default.existsSync(rcFilePath = _path2.default.resolve(cwd, options.rcFile)))) {
    cwd = _path2.default.resolve(cwd, '..');

    var _path$parse = _path2.default.parse(cwd),
        root = _path$parse.root,
        dir = _path$parse.dir;

    if (root === dir) {
      break;
    }
  }
  if (_fsExtra2.default.existsSync(rcFilePath)) {
    var _path$parse2 = _path2.default.parse(rcFilePath),
        _dir = _path$parse2.dir;

    cwd = _dir;
  } else {
    cwd = process.cwd();
  }

  if (rcFileExists) {
    var optionKeys = Object.keys(_options2.optionSchema.properties);
    options = (0, _lodash.assign)(options, (0, _lodash.pick)(require(rcFilePath), optionKeys), (0, _lodash.pick)(inputOptions, optionKeys));
  }

  var _options = options,
      files = _options.files,
      suffix = _options.suffix,
      rcFile = _options.rcFile,
      verbose = _options.verbose,
      injectors = _options.injectors;


  if (verbose) {
    if (rcFileExists) {
      console.log('Using config file', rcFilePath);
    } else {
      console.log('Not using config file.');
    }
  }
  if (verbose) console.log('Options:', options);

  if (!(0, _options2.validateOptions)(options)) throw new Error(_schema2.default.errorsText());

  if (verbose) console.log('Options:', options);

  files.forEach(function (file) {
    var templateFileName = file + '.' + suffix;
    var templateFilePath = _path2.default.resolve(cwd, templateFileName);
    if (extensions.every(function (e) {
      return !_fsExtra2.default.existsSync(templateFilePath + '.' + e);
    })) {
      console.log('cannot find', templateFilePath);
      return;
    }
    console.log('processing', templateFilePath);
    var template = require(templateFilePath);

    var injected = (0, _injectEnv2.default)(template, {
      defaultValue: '',
      env: injectors.reduce(function (a, b) {
        return Object.assign(a, b());
      }, {})
    });

    if (verbose) {
      console.log('injected', templateFileName);
      console.log(injected);
    }

    _fsExtra2.default.writeJSONSync(_path2.default.resolve(cwd, file + '.json'), injected, {
      spaces: 2
    });
  });
};

module.exports = jsonInjector;